{-# LANGUAGE FlexibleContexts #-}
module Utils where

import qualified Control.Monad.State as SM
import Data.Map (Map)
import qualified Data.Map as M

assumeJust :: Maybe a -> a
assumeJust (Just a) = a
assumeJust _ = error "assumeJust failed"

assumeRight :: Either l r -> r
assumeRight (Right x) = x
assumeRight _ = error "assumeRight failed"

getId :: (Ord k, SM.MonadState (Map k a, a) m, Enum a) => k -> m a
getId v = do
    (m, max) <- SM.get
    case M.lookup v m of
        Just x -> return x
        Nothing -> do
            let max' = succ max
            let m' = M.insert v max m
            SM.put (m', max')
            return max

buildMultiMap :: Ord a => [(a, b)] -> Map a [b]
buildMultiMap = foldr (\(a, b) -> M.insertWith (++) a [b]) M.empty

containsDuplicates :: Ord a => [a] -> Bool
containsDuplicates = any (>1) . M.elems . M.fromListWith (+) . map (flip (,) 1)
